"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const memory_cache_1 = require("./memory-cache");
const ms_1 = __importDefault(require("ms"));
const cache_then_network_1 = require("./cache-then-network");
var CACHING_STRATEGY;
(function (CACHING_STRATEGY) {
    CACHING_STRATEGY["CacheThenNetwork"] = "CacheThenNetwork";
    CACHING_STRATEGY["NetWorkThenCache"] = "NetWorkThenCache";
})(CACHING_STRATEGY || (CACHING_STRATEGY = {}));
exports.CACHING_STRATEGY = CACHING_STRATEGY;
const defaultCachingDuration = 60000;
const cachingStrategyImplementations = {
    [CACHING_STRATEGY.CacheThenNetwork]: cache_then_network_1.CacheThenNetwork
};
class CacheFactory {
    constructor() {
        this.plugins = {};
        this.plugins['memory'] = memory_cache_1.MemoryCache;
    }
    create(configuration) {
        if (configuration === true)
            configuration = {};
        const plugin = this.getPlugin(configuration.plugin);
        const cacheDuration = this.parseMs(configuration.duration);
        const strategy = this.getStrategy(configuration.strategy);
        const cacheWithCookie = configuration.cacheWithCookie || false;
        return new cachingStrategyImplementations[strategy](plugin, cacheWithCookie, cacheDuration);
    }
    getPlugin(plugin = 'memory') {
        const cachePlugin = (this.plugins[plugin] || this.plugins['memory']);
        if ('get' in cachePlugin && 'set' in cachePlugin) {
            return cachePlugin;
        }
        else {
            return new cachePlugin();
        }
    }
    parseMs(duration) {
        if (!duration)
            return defaultCachingDuration;
        switch (typeof duration) {
            case 'string':
                return ms_1.default(duration);
            case 'number':
                return duration;
            default:
                throw new Error('Unknown type provided for cache duration');
        }
    }
    getStrategy(strategy) {
        if (typeof strategy === 'undefined' || typeof CACHING_STRATEGY[strategy] === 'undefined') {
            return CACHING_STRATEGY.CacheThenNetwork;
        }
        return CACHING_STRATEGY[strategy];
    }
    register(name, plugin) {
        this.plugins[name] = plugin;
    }
}
exports.CacheFactory = CacheFactory;
//# sourceMappingURL=cache-factory.js.map